@@ -78,10 +78,10 @@ struct sun_disklabel {
 
 static inline uint16_t sun_pt_checksum(const struct sun_disklabel *label)
 {
-	uint16_t *ptr = ((uint16_t *) (label + 1)) - 1;
+	const uint16_t *ptr = ((const uint16_t *) (label + 1)) - 1;
 	uint16_t sum;
 
-	for (sum = 0; ptr >= ((uint16_t *) label);)
+	for (sum = 0; ptr >= ((const uint16_t *) label);)
 		sum ^= *ptr--;
 
 	return sum;
