@@ -13,27 +13,35 @@
   /* Fallback for types */
 # define wchar_t char
 # define wint_t int
-# define WEOF EOF
-  /* Fallback for input operations */
+# ifndef WEOF
+#  define WEOF EOF
+# endif
+
+ /* Fallback for input operations */
 # define fgetwc fgetc
 # define getwc getc
 # define getwchar getchar
 # define fgetws fgets
+
   /* Fallback for output operations */
 # define fputwc fputc
 # define putwc putc
 # define putwchar putchar
 # define fputws fputs
+
   /* Fallback for character classification */
 # define iswgraph isgraph
 # define iswprint isprint
 # define iswspace isspace
+
   /* Fallback for string functions */
 # define wcschr strchr
 # define wcsdup strdup
 # define wcslen strlen
 # define wcspbrk strpbrk
 
-# define wcwidth(c) 1
+# define wcwidth(c) (1)
+# define wmemset memset
+# define ungetwc ungetc
 
 #endif /* HAVE_WIDECHAR */
