@@ -91,18 +91,9 @@ blkdev_get_size(int fd, unsigned long long *bytes)
 #endif
 
 #ifdef BLKGETSIZE64
-	{
-#ifdef __linux__
-		int ver = get_linux_version();
-
-		/* kernels 2.4.15-2.4.17, had a broken BLKGETSIZE64 */
-		if (ver >= KERNEL_VERSION (2,6,0) ||
-		   (ver >= KERNEL_VERSION (2,4,18) && ver < KERNEL_VERSION (2,5,0)))
+	if (ioctl(fd, BLKGETSIZE64, bytes) >= 0)
+		return 0;
 #endif
-			if (ioctl(fd, BLKGETSIZE64, bytes) >= 0)
-				return 0;
-	}
-#endif /* BLKGETSIZE64 */
 
 #ifdef BLKGETSIZE
 	{
@@ -253,7 +244,7 @@ int blkdev_is_misaligned(int fd)
 	if (ioctl(fd, BLKALIGNOFF, &aligned) < 0)
 		return 0;			/* probably kernel < 2.6.32 */
 	/*
-	 * Note that kernel returns -1 as alignement offset if no compatible
+	 * Note that kernel returns -1 as alignment offset if no compatible
 	 * sizes and alignments exist for stacked devices
 	 */
 	return aligned != 0 ? 1 : 0;
@@ -406,4 +397,4 @@ main(int argc, char **argv)
 
 	return EXIT_SUCCESS;
 }
-#endif /* TEST_PROGRAM */
+#endif /* TEST_PROGRAM_BLKDEV */
