@@ -47,6 +47,9 @@ UL_DEBUG_DECLARE_MASK(libfdisk);
 #define ON_DBG(m, x)	__UL_DBG_CALL(libfdisk, LIBFDISK_DEBUG_, m, x)
 #define DBG_FLUSH	__UL_DBG_FLUSH(libfdisk, LIBFDISK_DEBUG_)
 
+#define UL_DEBUG_CURRENT_MASK	UL_DEBUG_MASK(libfdisk)
+#include "debugobj.h"
+
 /*
  * NLS -- the library has to be independent on main program, so define
  * UL_TEXTDOMAIN_EXPLICIT before you include nls.h.
@@ -279,6 +282,9 @@ struct fdisk_label {
 
 	int			flags;		/* FDISK_LABEL_FL_* flags */
 
+	struct fdisk_geometry	geom_min;	/* minimal geometry */
+	struct fdisk_geometry	geom_max;	/* maximal geometry */
+
 	unsigned int		changed:1,	/* label has been modified */
 				disabled:1;	/* this driver is disabled at all */
 
@@ -331,7 +337,8 @@ struct fdisk_ask {
 			uint64_t	unit;		/* unit for offsets */
 			const char	*range;		/* by library generated list */
 			unsigned int	relative :1,
-					inchars  :1;
+					inchars  :1,
+					wrap_negative	:1;
 		} num;
 		/* FDISK_ASKTYPE_{WARN,WARNX,..} */
 		struct ask_print {
@@ -358,6 +365,7 @@ struct fdisk_ask {
 struct fdisk_context {
 	int dev_fd;         /* device descriptor */
 	char *dev_path;     /* device path */
+	char *dev_model;    /* on linux /sys/block/<name>/device/model or NULL */
 	struct stat dev_st; /* stat(2) result */
 
 	int refcount;
@@ -380,6 +388,8 @@ struct fdisk_context {
 		     protect_bootbits : 1,	/* don't zeroize first sector */
 		     pt_collision : 1,		/* another PT detected by libblkid */
 		     no_disalogs : 1,		/* disable dialog-driven partititoning */
+		     dev_model_probed : 1,	/* tried to read from sys */
+		     private_fd : 1,		/* open by libfdisk */
 		     listonly : 1;		/* list partition, nothing else */
 
 	char *collision;			/* name of already existing FS/PT */
@@ -485,6 +495,7 @@ int fdisk_ask_number_set_high(struct fdisk_ask *ask, uint64_t high);
 int fdisk_ask_number_set_base(struct fdisk_ask *ask, uint64_t base);
 int fdisk_ask_number_set_unit(struct fdisk_ask *ask, uint64_t unit);
 int fdisk_ask_number_is_relative(struct fdisk_ask *ask);
+int fdisk_ask_number_set_wrap_negative(struct fdisk_ask *ask, int wrap_negative);
 int fdisk_ask_menu_set_default(struct fdisk_ask *ask, int dfl);
 int fdisk_ask_menu_add_item(struct fdisk_ask *ask, int key,
 			const char *name, const char *desc);
