@@ -281,7 +281,7 @@ static PyObject *Table_find_srcpath(TableObject *self, PyObject *args, PyObject
 #define Table_find_tag_HELP "find_tag(tag, val, [direction])\n\n" \
 		"Try to lookup an entry in given tab, first attempt is lookup by tag and\n" \
 		"val, for the second attempt the tag is evaluated (converted to the device\n" \
-		"name) and Tab.find_srcpath() is preformed. The second attempt is not\n" \
+		"name) and Tab.find_srcpath() is performed. The second attempt is not\n" \
 		"performed when tb cache is not set (not implemented yet).\n" \
 		"\n" \
 		"Returns a tab entry or NULL."
@@ -733,38 +733,38 @@ PyTypeObject TableType = {
 	sizeof(TableObject), /*tp_basicsize*/
 	0, /*tp_itemsize*/
 	(destructor)Table_destructor, /*tp_dealloc*/
-	0, /*tp_print*/
-	0, /*tp_getattr*/
-	0, /*tp_setattr*/
-	0, /*tp_compare*/
+	NULL, /*tp_print*/
+	NULL, /*tp_getattr*/
+	NULL, /*tp_setattr*/
+	NULL, /*tp_compare*/
 	(reprfunc) Table_repr, /*tp_repr*/
-	0, /*tp_as_number*/
-	0, /*tp_as_sequence*/
-	0, /*tp_as_mapping*/
-	0, /*tp_hash */
-	0, /*tp_call*/
-	0, /*tp_str*/
-	0, /*tp_getattro*/
-	0, /*tp_setattro*/
-	0, /*tp_as_buffer*/
+	NULL, /*tp_as_number*/
+	NULL, /*tp_as_sequence*/
+	NULL, /*tp_as_mapping*/
+	NULL, /*tp_hash */
+	NULL, /*tp_call*/
+	NULL, /*tp_str*/
+	NULL, /*tp_getattro*/
+	NULL, /*tp_setattro*/
+	NULL, /*tp_as_buffer*/
 	Py_TPFLAGS_DEFAULT | Py_TPFLAGS_BASETYPE, /*tp_flags*/
 	Table_HELP, /* tp_doc */
-	0, /* tp_traverse */
-	0, /* tp_clear */
-	0, /* tp_richcompare */
+	NULL, /* tp_traverse */
+	NULL, /* tp_clear */
+	NULL, /* tp_richcompare */
 	0, /* tp_weaklistoffset */
-	0, /* tp_iter */
-	0, /* tp_iternext */
+	NULL, /* tp_iter */
+	NULL, /* tp_iternext */
 	Table_methods, /* tp_methods */
 	Table_members, /* tp_members */
 	Table_getseters, /* tp_getset */
-	0, /* tp_base */
-	0, /* tp_dict */
-	0, /* tp_descr_get */
-	0, /* tp_descr_set */
+	NULL, /* tp_base */
+	NULL, /* tp_dict */
+	NULL, /* tp_descr_get */
+	NULL, /* tp_descr_set */
 	0, /* tp_dictoffset */
 	(initproc)Table_init, /* tp_init */
-	0, /* tp_alloc */
+	NULL, /* tp_alloc */
 	Table_new, /* tp_new */
 };
 
