@@ -1,8 +1,13 @@
+/* SPDX-License-Identifier: LGPL-2.1-or-later */
 /*
- * Copyright (C) 2008,2009,2012 Karel Zak <kzak@redhat.com>
+ * This file is part of libmount from util-linux project.
  *
- * This file may be redistributed under the terms of the
- * GNU Lesser General Public License.
+ * Copyright (C) 2009-2018 Karel Zak <kzak@redhat.com>
+ *
+ * libmount is free software; you can redistribute it and/or modify it
+ * under the terms of the GNU Lesser General Public License as published by
+ * the Free Software Foundation; either version 2.1 of the License, or
+ * (at your option) any later version.
  */
 
 /**
@@ -346,7 +351,9 @@ int mnt_optstr_deduplicate_option(char **optstr, const char *name)
 			end = ol.end;
 			opt = end && *end ? end + 1 : NULL;
 		}
-	} while (rc == 0 && opt && *opt);
+		if (opt == NULL)
+			break;
+	} while (rc == 0 && *opt);
 
 	return rc < 0 ? rc : begin ? 0 : 1;
 }
@@ -382,7 +389,7 @@ insert_value(char **str, char *pos, const char *substr, char **next)
 	size_t strsz = strlen(*str);
 	size_t possz = strlen(pos);
 	size_t posoff;
-	char *p = NULL;
+	char *p;
 	int sep;
 
 	/* is it necessary to prepend '=' before the substring ? */
@@ -800,14 +807,17 @@ int mnt_optstr_apply_flags(char **optstr, unsigned long flags,
 					if (rc)
 						goto err;
 				}
-				if (!(ent->mask & MNT_INVERT))
+				if (!(ent->mask & MNT_INVERT)) {
 					fl &= ~ent->id;
+					if (ent->id & MS_REC)
+						fl |= MS_REC;
+				}
 			}
 		}
 	}
 
-	/* add missing options */
-	if (fl) {
+	/* add missing options (but ignore fl if contains MS_REC only) */
+	if (fl && fl != MS_REC) {
 		const struct libmnt_optmap *ent;
 		char *p;
 
