@@ -118,8 +118,8 @@ static int call_daemon(const char *socket_path, int op, char *buf,
 	}
 
 	srv_addr.sun_family = AF_UNIX;
-	strncpy(srv_addr.sun_path, socket_path, sizeof(srv_addr.sun_path));
-	srv_addr.sun_path[sizeof(srv_addr.sun_path) - 1] = '\0';
+	assert(strlen(socket_path) < sizeof(srv_addr.sun_path));
+	xstrncpy(srv_addr.sun_path, socket_path, sizeof(srv_addr.sun_path));
 
 	if (connect(s, (const struct sockaddr *) &srv_addr,
 		    sizeof(struct sockaddr_un)) < 0) {
@@ -252,8 +252,8 @@ static int create_socket(struct uuidd_cxt_t *uuidd_cxt,
 	 * Create the address we will be binding to.
 	 */
 	my_addr.sun_family = AF_UNIX;
-	strncpy(my_addr.sun_path, socket_path, sizeof(my_addr.sun_path));
-	my_addr.sun_path[sizeof(my_addr.sun_path) - 1] = '\0';
+	assert(strlen(socket_path) < sizeof(my_addr.sun_path));
+	xstrncpy(my_addr.sun_path, socket_path, sizeof(my_addr.sun_path));
 	unlink(socket_path);
 	save_umask = umask(0);
 	if (bind(s, (const struct sockaddr *) &my_addr,
@@ -298,7 +298,9 @@ static void timeout_handler(int sig __attribute__((__unused__)),
 			    siginfo_t * info,
 			    void *context __attribute__((__unused__)))
 {
+#ifdef HAVE_TIMER_CREATE
 	if (info->si_code == SI_TIMER)
+#endif
 		errx(EXIT_FAILURE, _("timed out"));
 }
 
@@ -311,7 +313,8 @@ static void server_loop(const char *socket_path, const char *pidfile_path,
 	uuid_t			uu;
 	char			reply_buf[1024], *cp;
 	char			op, str[UUID_STR_LEN];
-	int			i, ns, len, num;
+	int			i, ns, len;
+	int			num;		/* intentionally uninitialized */
 	int			s = 0;
 	int			fd_pidfile = -1;
 	int			ret;
@@ -327,18 +330,18 @@ static void server_loop(const char *socket_path, const char *pidfile_path,
 	if (!uuidd_cxt->no_sock)	/* no_sock implies no_fork and no_pid */
 #endif
 	{
-		static timer_t t_id;
+		struct ul_timer timer;
 		struct itimerval timeout;
 
 		memset(&timeout, 0, sizeof timeout);
 		timeout.it_value.tv_sec = 30;
-		if (setup_timer(&t_id, &timeout, &timeout_handler))
+		if (setup_timer(&timer, &timeout, &timeout_handler))
 			err(EXIT_FAILURE, _("cannot set up timer"));
 		if (pidfile_path)
 			fd_pidfile = create_pidfile(uuidd_cxt, pidfile_path);
 		ret = call_daemon(socket_path, UUIDD_OP_GETPID, reply_buf,
 				  sizeof(reply_buf), 0, NULL);
-		cancel_timer(&t_id);
+		cancel_timer(&timer);
 		if (ret > 0) {
 			if (!uuidd_cxt->quiet)
 				warnx(_("uuidd daemon is already running at pid %s"),
@@ -573,7 +576,7 @@ int main(int argc, char **argv)
 	setlocale(LC_ALL, "");
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
-	atexit(close_stdout);
+	close_stdout_atexit();
 
 	while ((c =
 		getopt_long(argc, argv, "p:s:T:krtn:PFSdqVh", longopts,
@@ -626,9 +629,9 @@ int main(int argc, char **argv)
 			uuidd_cxt.timeout = strtou32_or_err(optarg,
 						_("failed to parse --timeout"));
 			break;
+
 		case 'V':
-			printf(UTIL_LINUX_VERSION);
-			return EXIT_SUCCESS;
+			print_version(EXIT_SUCCESS);
 		case 'h':
 			usage();
 		default:
@@ -636,6 +639,9 @@ int main(int argc, char **argv)
 		}
 	}
 
+	if (strlen(socket_path) >= sizeof(((struct sockaddr_un *)0)->sun_path))
+		errx(EXIT_FAILURE, _("socket name too long: %s"), socket_path);
+
 	if (!no_pid && !pidfile_path)
 		pidfile_path = UUIDD_PIDFILE_PATH;
 
