@@ -7,23 +7,23 @@
 int main(void)
 {
 	int i, ret;
-	struct MD5Context ctx;
-	unsigned char digest[MD5LENGTH];
+	struct UL_MD5Context ctx;
+	unsigned char digest[UL_MD5LENGTH];
 	unsigned char buf[BUFSIZ];
 
-	MD5Init( &ctx );
+	ul_MD5Init( &ctx );
 
 	while(!feof(stdin) && !ferror(stdin)) {
 		ret = fread(buf, 1, sizeof(buf), stdin);
 		if (ret)
-			MD5Update( &ctx, buf, ret );
+			ul_MD5Update( &ctx, buf, ret );
 	}
 
 	fclose(stdin);
-	MD5Final( digest, &ctx );
+	ul_MD5Final( digest, &ctx );
 
-	for (i = 0; i < MD5LENGTH; i++)
+	for (i = 0; i < UL_MD5LENGTH; i++)
 		printf( "%02x", digest[i] );
-	printf("  -\n");
+	printf("\n");
 	return 0;
 }
